"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Runtime manager is a view service that supports user interaction with the running platforms.

"""

import asyncio

from requests import Response
from pyegeria.utils import body_slimmer
from pyegeria._client import Client
from pyegeria import TEMPLATE_GUIDS, max_paging_size,default_time_out
from pyegeria._exceptions import InvalidParameterException
from pyegeria._globals import NO_ELEMENTS_FOUND

class RuntimeManager(Client):
    """
    Client to issue Runtime status requests.

    Attributes:

        view_server : str
                Name of the server to use.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None
        token: str, optional
            Bearer token

    Methods:

    """

    def __init__(
        self,
        view_server: str,
        platform_url: str,
        user_id: str,
        user_pwd: str = None,
        token: str = None,
        time_out: int = default_time_out,
    ):
        self.view_server = view_server
        self.time_out = time_out
        Client.__init__(self, view_server, platform_url, user_id, user_pwd, token=token)
        self.runtime_command_root = f"{self.platform_url}/servers/{self.view_server}/api/open-metadata/runtime-manager"
        # self.platform_guid = "44bf319f-1e41-4da1-b771-2753b92b631a"  # this is platform @ 9443 from the core content archive
        self.platform_guid = None
        self.default_platform_name = (
            "Default Local OMAG Server Platform"  # this from the core content archive
        )

    #
    #   Cohorts
    #
    async def _async_connect_to_cohort(
        self,
        cohort_name: str,
        server_guid: str = None,
        qualified_name: str = None,
    ) -> None:
        """A new server needs to register the metadataCollectionId for its metadata repository with the other servers
            in the open metadata repository. It only needs to do this once and uses a timestamp to record that the
            registration event has been sent. If the server has already registered in the past, it sends a
            reregistration request.  Async version.

        https://egeria-project.org/concepts/cohort-member/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid must be.
        cohort_name : str
            Name of the cohort to join

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(server_guid, qualified_name=qualified_name)
        url = (
            f"{self.runtime_command_root}/cohort-members/"
            f"{server_guid}/cohorts/{cohort_name}/connect"
        )
        await self._async_make_request("GET", url)
        return

    def connect_to_cohort(
        self,
        cohort_name: str,
        server_guid: str = None,
        qualified_name: str = None,
    ) -> None:
        """A new server needs to register the metadataCollectionId for its metadata repository with the other servers
            in the open metadata repository. It only needs to do this once and uses a timestamp to record that the
            registration event has been sent. If the server has already registered in the past, it sends a
            reregistration request.

        https://egeria-project.org/concepts/cohort-member/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid must be.
        cohort_name: str
            Name of the cohort to join

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_connect_to_cohort(cohort_name, server_guid, qualified_name)
        )
        return

    async def _async_disconnect_from_cohort(
        self,
        cohort_name: str,
        server_guid: str = None,
        qualified_name: str = None,
    ) -> None:
        """Disconnect communications from a specific cohort.  Async version.

            https://egeria-project.org/concepts/cohort-member/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid must be.
        cohort_name : str
            Name of the cohort to join

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(server_guid, qualified_name=qualified_name)
        url = (
            f"{self.runtime_command_root}/runtime-manager/cohort-members/"
            f"{server_guid}/cohorts/{cohort_name}/disconnect"
        )
        await self._async_make_request("GET", url)
        return

    def disconnect_from_cohort(
        self,
        cohort_name: str,
        server_guid: str = None,
        qualified_name: str = None,
    ) -> None:
        """Disconnect communications from a specific cohort.

            https://egeria-project.org/concepts/cohort-member/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid must be.
        cohort_name: str
            Name of the cohort to join

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_disconnect_from_cohort(cohort_name, server_guid, qualified_name)
        )
        return

    async def _async_unregister_from_cohort(
        self,
        cohort_name: str,
        server_guid: str = None,
        qualified_name: str = None,
    ) -> None:
        """Unregister from a specific cohort and disconnect from cohort communications.  Async version.

            https://egeria-project.org/concepts/cohort-member/

        Parameters
        ----------
         server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid must be.
        cohort_name : str
            Name of the cohort to join

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(server_guid, qualified_name=qualified_name)
        url = (
            f"{self.runtime_command_root}/cohort-members/"
            f"{server_guid}/cohorts/{cohort_name}/unregister"
        )
        await self._async_make_request("GET", url)
        return

    def unregister_from_cohort(
        self,
        cohort_name: str,
        server_guid: str = None,
        qualified_name: str = None,
    ) -> None:
        """Unregister from a specific cohort and disconnect from cohort communications.
            https://egeria-project.org/concepts/cohort-member/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid must be.
        cohort_name: str
            Name of the cohort to join

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_disconnect_from_cohort(cohort_name, server_guid, qualified_name)
        )
        return

    #
    #   Governance Engines
    #

    async def _async_refresh_gov_eng_config(
        self,
        gov_engine_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Request that the governance engine refresh its configuration by calling the metadata server. This request is
            useful if the metadata server has an outage, particularly while the governance server is initializing.
            This request just ensures that the latest configuration is in use. If gov_engine_name is None, all engines
            will be refreshed. Async version.

            https://egeria-project.org/concepts/governance-engine-definition/

        Parameters
        ----------
        gov_engine_name: str, default = None
            If None, then all engines will be refreshed - this is the normal case. If an engine is specified only this
            engine will be refreshed.
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or display_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, opt, default is None.
                        Identity of the server to act on. Either the server_guid , qualified_name, or server_name must
                        be provided.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid, display_name, "name", qualified_name, tech_type="Engine Host"
        )
        if gov_engine_name is None:
            url = (
                f"{self.runtime_command_root}/engine-hosts/"
                f"{server_guid}/governance-engines/refresh-config"
            )
        else:
            url = (
                f"{self.runtime_command_root}/engine-hosts/"
                f"{server_guid}/governance-engines/{gov_engine_name}/refresh-config"
            )
        await self._async_make_request("GET", url)
        return

    def refresh_gov_eng_config(
        self,
        gov_engine_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Request that the governance engine refresh its configuration by calling the metadata server. This request is
            useful if the metadata server has an outage, particularly while the governance server is initializing.
            This request just ensures that the latest configuration is in use. If gov_engine_name is None, all engines
            will be refreshed. Async version.

            https://egeria-project.org/concepts/governance-engine-definition/

        Parameters
        ----------
        gov_engine_name: str, default = None
            If None, then all engines will be refreshed - this is the normal case. If an engine is specified only this
            engine will be refreshed.
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or display_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, opt, default is None.
                        Identity of the server to act on. Either the server_guid , qualified_name, or server_name must
                        be provided.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_refresh_gov_eng_config(
                gov_engine_name, server_guid, display_name, qualified_name
            )
        )
        return

    #
    #   Integration Connector Methods
    #
    async def _async_get_integ_connector_config_properties(
        self,
        connector_name: str,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> dict | str:
        """Retrieve the configuration properties of the named integration connector running in the integration daemon.
            Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str
            Name of the integration connector to retrieve properties for.

        Returns
        -------
           Dict of the connector configuration properties.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "qualifiedName",
            qualified_name,
            "Integration Daemon",
        )
        url = (
            f"{self.runtime_command_root}/integration-daemons/"
            f"{server_guid}/integration-connectors/{connector_name}/configuration-properties"
        )
        response = await self._async_make_request("GET", url)
        return response.json().get("properties", "No pproperties found")

    def get_integ_connector_config_properties(
        self,
        connector_name: str,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> dict | str:
        """Retrieve the configuration properties of the named integration connector running in the integration daemon.
            Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        connector_name : str
            Name of the integration connector to retrieve properties for.
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_integ_connector_config_properties(
                connector_name, server_guid, display_name, qualified_name
            )
        )
        return response

    async def _async_update_connector_configuration(
        self,
        connector_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        merge_update: bool = True,
        config_properties: dict = None,
    ) -> None:
        """Update the configuration properties of the integration connectors, or specific integration connector
            if a connector name is supplied.  This update is in memory and will not persist over a server restart.
            Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        connector_name : str, default = None
            Name of the integration connector to update properties for. If none, all connectors will be updated.
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.

        merge_update : bool, optional, default = False
            Specifies whether properties should be over-written or completely replace existing properties. If False
            the values will be replaced; if True, they will be merged.
        config_properties : dict, optional, default = None
            A dict of Property Name, Property Value pairs.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "qualifiedName",
            qualified_name,
            "Integration Daemon",
        )
        url = (
            f"{self.runtime_command_root}/integration-daemons/"
            f"{server_guid}/integration-connectors/configuration-properties"
        )

        body = {
            "class": "ConnectorConfigPropertiesRequestBody",
            "connectorName": connector_name,
            "mergeUpdate": merge_update,
            "configurationProperties": config_properties,
        }
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def update_connector_configuration(
        self,
        connector_name: str,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        merge_update: bool = False,
        config_properties: dict = None,
    ) -> None:
        """Update the configuration properties of the integration connectors, or specific integration connector
            if a connector name is supplied.  This update is in memory and will not persist over a server restart.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str
            Name of the integration connector to retrieve properties for.
        merge_update : bool, optional, default = False
            Specifies whether properties should be over-written or completely replace existing properties. If False
            the values will be replaced; if True, they will be merged.
        config_properties : dict, optional, default = None
            A dict of Property Name, Property Value pairs.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_connector_configuration(
                connector_name,
                server_guid,
                display_name,
                qualified_name,
                merge_update,
                config_properties,
            )
        )
        return

    async def _async_update_endpoint_address(
        self,
        connector_name: str,
        endpoint_address: str,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Update the endpoint network address for a specific integration connector.  Typically used for discovery.
            This update is in memory and will not persist over a server restart. Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str
            Name of the integration connector to retrieve properties for.
        endpoint_address : str
            Specifies the new network endpoint address. This is the full address string - can include protocol,
            port, operation, etc.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid, display_name, "qualifiedName", qualified_name, "Connection"
        )
        url = (
            f"{self.runtime_command_root}/integration-daemons/"
            f"{server_guid}/integration-connectors/{connector_name}/endpoint-network-address"
        )

        body = {
            "class": "StringRequestBody",
            "string": endpoint_address,
        }
        await self._async_make_request("POST", url, body)
        return

    def update_endpoint_address(
        self,
        connector_name: str,
        endpoint_address: str,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Update the endpoint network address for a specific integration connector.  Typically used for discovery.
            This update is in memory and will not persist over a server restart. Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str
            Name of the integration connector to retrieve properties for.
        endpoint_address : str
            Specifies the new network endpoint address. This is the full address string - can include protocol,
            port, operation, etc.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_update_endpoint_address(
                connector_name,
                endpoint_address,
                server_guid,
                display_name,
                qualified_name,
            )
        )
        return

    async def _async_refresh_integration_connectors(
        self,
        connector_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Issue a refresh() request on all connectors running in the integration daemon, or a specific connector if
            the connector name is specified. Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str, opt
            Name of the integration connector to retrieve properties for. If None, all connectors refreshed.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "qualifiedName",
            qualified_name,
            "Integration Daemon",
        )
        url = (
            f"{self.runtime_command_root}/integration-daemons/"
            f"{server_guid}/integration-connectors/refresh"
        )

        body = {
            "class": "NameRequestBody",
            "string": connector_name,
        }
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def refresh_integration_connectors(
        self,
        connector_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Issue a refresh() request on all connectors running in the integration daemon, or a specific connector if
            the connector name is specified.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str, opt
            Name of the integration connector to retrieve properties for. If None, all connectors refreshed.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_refresh_integration_connectors(
                connector_name, server_guid, display_name, qualified_name
            )
        )
        return

    async def _async_restart_integration_connectors(
        self,
        connector_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Issue a restart() request on all connectors running in the integration daemon, or a specific connector if
            the connector name is specified. Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str, opt
            Name of the integration connector to retrieve properties for. If None, all connectors restarted.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "qualifiedName",
            qualified_name,
            "Integration Daemon",
        )
        url = (
            f"{self.runtime_command_root}/integration-daemons/"
            f"{server_guid}/integration-connectors/restart"
        )

        body = {
            "class": "NameRequestBody",
            "string": connector_name,
        }
        await self._async_make_request("POST", url, body_slimmer(body))
        return

    def restart_integration_connectors(
        self,
        connector_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Issue a restart() request on all connectors running in the integration daemon, or a specific connector if
            the connector name is specified.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        connector_name : str, opt
            Name of the integration connector to retrieve properties for. If None, all connectors restarted.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_restart_integration_connectors(
                connector_name, server_guid, display_name, qualified_name
            )
        )
        return

    async def _async_refresh_integ_group_config(
        self,
        integ_group_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Request that the integration group refresh its configuration by calling the metadata access server.
            Changes to the connector configuration will result in the affected connectors being restarted.
            This request is useful if the metadata access server has an outage, particularly while the integration
            daemon is initializing. This request just ensures that the latest configuration is in use. Async version.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        integ_group_name : str, opt, default = None
            Name of the integration group to refresh. If None, all groups are refreshed.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "qualifiedName",
            qualified_name,
            "Integration Daemon",
        )
        url = (
            f"{self.runtime_command_root}/integration-daemons/"
            f"{server_guid}/integration-groups/{integ_group_name}/refresh-config"
        )

        await self._async_make_request("GET", url)
        return

    def refresh_integ_group_config(
        self,
        integ_group_name: str = None,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Request that the integration group refresh its configuration by calling the metadata access server.
            Changes to the connector configuration will result in the affected connectors being restarted.
            This request is useful if the metadata access server has an outage, particularly while the integration
            daemon is initializing. This request just ensures that the latest configuration is in use.

            https://egeria-project.org/concepts/integration-connector/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        integ_group_name : str, opt, default = None
            Name of the integration group to refresh. If None, all groups are refreshed.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_refresh_integ_group_config(
                integ_group_name, server_guid, display_name, qualified_name
            )
        )
        return

    #
    # Open Lineage & Archives
    #
    async def _async_publish_open_lineage_event(
        self,
        ol_event: dict,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Send an Open Lineage event to the integration daemon. It will pass it on to the integration connectors that
            have registered a listener for open lineage events. Async version.

            https://egeria-project.org/features/lineage-management/overview/#the-openlineage-standard

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        ol_event : dict
            Dict containing the user specified Open Lineage event.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "name",
            qualified_name,
        )
        url = (
            f"{self.runtime_command_root}/integration-daemons/"
            f"{server_guid}/integration-daemons/{server_guid}/open-lineage-events/publish"
        )

        await self._async_make_request("POST", url, ol_event)
        return

    def publish_open_lineage_event(
        self,
        ol_event: dict,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
    ) -> None:
        """Send an Open Lineage event to the integration daemon. It will pass it on to the integration connectors that
            have registered a listener for open lineage events.

            https://egeria-project.org/features/lineage-management/overview/#the-openlineage-standard

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        ol_event : dict
            Dict containing the user specified Open Lineage event.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_publish_open_lineage_event(ol_event, server_guid, display_name)
        )
        return

    async def _async_add_archive_content(
        self,
        archive_content: dict,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        time_out: int = 60,
    ) -> None:
        """An open metadata archive contains metadata types and instances.
            This operation loads the supplied open metadata archive into the local repository. It can be used with OMAG
            servers that are of type Open Metadata Store. Async version.

                https://egeria-project.org/concepts/open-metadata-archives/

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        archive_content : dict
            A dict containing the content of the archive to load.
        time_out : int, optional, default = 60 seconds
            Timeout for the REST call.

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "qualifiedName",
            qualified_name,
            "Metadata Access Server",
        )
        url = (
            f"{self.runtime_command_root}/metadata-access-stores/{server_guid}/instance/load/open-metadata-archives/"
            f"archive-content"
        )

        await self._async_make_request("POST", url, archive_content, time_out=time_out)
        return

    def add_archive_content(
        self,
        archive_content: dict,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        time_out: int = 60,
    ) -> None:
        """An open metadata archive contains metadata types and instances.
            This operation loads the supplied open metadata archive into the local repository. It can be used with OMAG
            servers that are of type Open Metadata Store.

                https://egeria-project.org/concepts/open-metadata-archives/

        Parameters
        ----------
        archive_content : dict
            A dict containing the content of the archive to load.
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        time_out : int, optional, default = 60 seconds
            Timeout for the REST call.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_add_archive_content(
                archive_content, server_guid, display_name, qualified_name, time_out
            )
        )
        return

    async def _async_add_archive_file(
        self,
        archive_file: str,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        time_out: int = 120,
    ) -> None:
        """Add a new open metadata archive to running OMAG Server's repository.
            An open metadata archive contains metadata types and instances.  This operation loads an open metadata archive
            that is stored in the named file.  It can be used with OMAG servers that are of type Open Metadata Store.
            Async version.

            https://egeria-project.org/concepts/open-metadata-archives/

        Parameters
        ----------
        archive_file: str
            Open metadata archive file to load.
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        time_out: int, optional
           Time out for the rest call.

        Returns
        -------
        Response
          None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid,
            display_name,
            "qualifiedName",
            qualified_name,
            "Metadata Access Server",
        )
        url = f"{self.runtime_command_root}/metadata-access-stores/{server_guid}/instance/load/open-metadata-archives/file"

        await self._async_make_request(
            "POST-DATA", url, archive_file, time_out=time_out
        )
        return

    def add_archive_file(
        self,
        archive_file: str,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        time_out: int = 120,
    ) -> None:
        """Add a new open metadata archive to running OMAG Server's repository.
            An open metadata archive contains metadata types and instances.  This operation loads an open metadata archive
            that is stored in the named file.  It can be used with OMAG servers that are of type Open Metadata Store.

            https://egeria-project.org/concepts/open-metadata-archives/

        Parameters
        ----------
        archive_file: str
            Open metadata archive file to load.
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid or server_name must be.
        time_out: int, optional, default = 60 seconds

        Returns
        -------
        Response
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_add_archive_file(
                archive_file, server_guid, display_name, qualified_name, time_out
            )
        )
        return

    #
    # Server & Platform admin
    #
    async def _async_shutdown_and_unregister_server(
        self,
        server_guid: str = None,
        qualified_name: str = None,
    ) -> None:
        """Shutdown the named OMAG server. The server will also be removed from any open metadata repository cohorts
            it has registered with.  Async version.

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name  must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid  must be.

        Returns
        -------
        Response
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid, property_name="qualifiedName", qualified_name=qualified_name
        )
        url = f"{self.runtime_command_root}/omag-servers/{server_guid}"

        await self._async_make_request("DELETE", url)

        return

    def shutdown_and_unregister_server(
        self, server_guid: str = None, qualified_name: str = None
    ) -> None:
        """Shutdown the named OMAG server. The server will also be removed from any open metadata repository cohorts
            it has registered with.

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name  must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid  must be.
        Returns
        -------
        Response
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_shutdown_and_unregister_server(server_guid, qualified_name)
        )
        return

    async def _async_activate_server_with_stored_config(
        self,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        timeout: int = 240,
    ) -> None:
        """Activate the named OMAG server using the appropriate configuration document found in the
            configuration store. Async version.

        https://egeria-project.org/concepts/configuration-document

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name or server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid or qualified_name must be.
        qualified_name: str, default = None
            Unique name of server to act on. If not specified, server_guid  must be.
        timeout: int, optional, default = 240 seconds

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid, display_name, "name", qualified_name
        )

        url = f"{self.runtime_command_root}/omag-servers/{server_guid}/instance"

        await self._async_make_request("POST", url, time_out=timeout)
        return

    def activate_server_with_stored_config(
        self,
        server_guid: str = None,
        display_name: str = None,
        qualified_name: str = None,
        timeout: int = 240,
    ) -> None:
        """Activate the named OMAG server using the appropriate configuration document found in the
            configuration store.

        https://egeria-project.org/concepts/configuration-document

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        display_name: str, default = None
            Name of server to act on. If not specified, server_guid must be.
        timeout: int, optional, default = 240 seconds

        Returns
        -------
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_activate_server_with_stored_config(
                server_guid, display_name, timeout
            )
        )
        return

    async def _async_shutdown_server(
        self, server_guid: str = None, qualified_name: str = None
    ) -> None:
        """Temporarily shutdown the named OMAG server. This server can be restarted as a later time.  Async version.

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name must be.
        qualified_name: str, default = None
            Qualified name of server to act on. If not specified, server_guid must be.

        Returns
        -------
        Response
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid, None, "qualifiedName", qualified_name
        )
        url = f"{self.runtime_command_root}/omag-servers/{server_guid}/instance"

        await self._async_make_request("DELETE", url)

        return

    def shutdown_server(
        self, server_guid: str = None, qualified_name: str = None
    ) -> None:
        """Temporarily shutdown the named OMAG server. This server can be restarted as a later time.

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, qualified_name must be.
        qualified_name: str, default = None
            Qualified name of server to act on. If not specified, server_guid must be.

        Returns
        -------
        Response
           None

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(
            self._async_shutdown_server(server_guid, qualified_name)
        )
        return

    def get_platforms_by_name(
        self,
        filter: str = None,
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of platforms with a particular name. The name is specified in the filter.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the display name or qualified name of the platforms to return information for. If the
            value is None, we will default to the default_platform_name that comes from the core content pack.
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException
        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_platforms_by_name(
                filter, effective_time, start_from, page_size
            )
        )
        return response

    async def _async_get_platforms_by_name(
        self,
        filter: str = None,
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of platforms with a particular name. The name is specified in the filter.  Async version.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the display name or qualified name of the platforms to return information for. If the
            value is None, we will default to the default_platform_name that comes from the core content pack.
        effective_time: str, optional
           Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.

        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        if filter is None:
            filter = self.default_platform_name

        url = (
            f"{self.runtime_command_root}/platforms/by-name?"
            f"startFrom={start_from}&pageSize={page_size}"
        )
        if effective_time is not None:
            body = {"filter": filter, "effectiveTime": effective_time}
        else:
            body = {"filter": filter}

        response = await self._async_make_request("POST", url, body)

        return response.json().get("elements", "No platforms found")

    async def _async_get_platforms_by_type(
        self,
        filter: str = None,
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of platforms with a particular deployed implementation type.  The value is specified in
            the filter. If it is null, or no request body is supplied, all platforms are returned.  Async version.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        if filter is None:
            filter = "OMAG Server Platform"

        url = (
            f"{self.runtime_command_root}/platforms/"
            f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}"
        )

        if effective_time is not None:
            body = {"filter": filter, "effectiveTime": effective_time}
        else:
            body = {"filter": filter}

        response = await self._async_make_request("POST", url, body)
        return response.json().get("elements", "No platforms found")

    def get_platforms_by_type(
        self,
        filter: str = None,
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of platforms with a particular deployed implementation type.  The value is specified in
            the filter. If it is null, or no request body is supplied, all platforms are returned.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_platforms_by_type(
                filter, effective_time, start_from, page_size
            )
        )
        return response

    async def _async_get_platform_templates_by_type(
        self,
        filter: str = None,
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of platform templates for a particular deployed implementation type.  The value is
            specified in the filter. If it is null, or no request body is supplied, all platforms are returned.
            Async version.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        if filter is None:
            filter = "OMAG Server Platform"

        url = (
            f"{self.runtime_command_root}/platforms/"
            f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}&getTemplates=true"
        )

        if effective_time is not None:
            body = {"filter": filter, "effectiveTime": effective_time}
        else:
            body = {"filter": filter}

        response = await self._async_make_request("POST", url, body)
        return response.json().get("elements", "No platforms found")

    def get_platform_templates_by_type(
        self,
        filter: str = None,
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of platform templates with a particular deployed implementation type.  The value is
            specified in the filter. If it is null, or no request body is supplied, all platforms are returned.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_platforms_by_type(
                filter, effective_time, start_from, page_size
            )
        )
        return response

    async def _async_get_platform_report(
        self, platform_guid: str = None, platform_name: str = None
    ) -> str | list:
        """Returns details about the running platform. Async version.

        Parameters
        ----------
        platform_guid : str
            The unique identifier for the platform. If not specified, platform_name must be.
        platform_name: str, default = None
            Name of server to act on. If not specified, platform_guid must be.

        Returns
        -------
        Response
           A json dict with the platform report.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        platform_guid = self.__get_guid__(platform_guid, platform_name, "name")
        url = f"{self.runtime_command_root}/platforms/{platform_guid}/report"

        response = await self._async_make_request("GET", url)

        return response.json().get("element", "No platforms found")

    def get_platform_report(
        self, platform_guid: str = None, platform_name: str = None
    ) -> str | list:
        """Returns details about the running platform.

        Parameters
        ----------
        platform_guid : str
            The unique identifier for the platform. If not specified, platform_name must be.
        platform_name: str, default = None
            Name of server to act on. If not specified, platform_guid must be.
        Returns
        -------
        Response
           A json dict with the platform report.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_platform_report(platform_guid, platform_name)
        )
        return response

    async def _async_get_platform_by_guid(
        self,
        platform_guid: str,
        effective_time: str = None,
    ) -> str | list:
        """Returns details about the platform's catalog entry (asset). Async version.

        Parameters
        ----------
        platform_guid : str
            Unique id of the platform to return details of.
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).


        Returns
        -------
        Response
           A list of json dict with the platform reports.

        """

        url = f"{self.runtime_command_root}/platforms/{platform_guid}"

        if effective_time is not None:
            body = {"effectiveTime": effective_time}
            response = await self._async_make_request("POST", url, body)

        else:
            response = await self._async_make_request("POST", url)

        return response.json().get("element", "No platforms found")

    def get_platform_by_guid(
        self,
        platform_guid: str,
        effective_time: str = None,
    ) -> str | list:
        """Returns details about the platform's catalog entry (asset).

        Parameters
        ----------
        platform_guid : str, opt
            Identity of the platform to return details about.
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_platform_by_guid(platform_guid, effective_time)
        )
        return response

    async def _async_get_server_by_guid(
        self,
        server_guid: str = None,
        effective_time: str = None,
    ) -> str | dict:
        """Returns details about the server's catalog entry (asset). Async version.

        Parameters
        ----------
        server_guid : str
            The unique identifier for the platform.
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        url = f"{self.runtime_command_root}/software-servers/{server_guid}"

        if effective_time is not None:
            body = {"effectiveTime": effective_time}
            response = await self._async_make_request("POST", url, body)

        else:
            response = await self._async_make_request("POST", url)

        return response.json().get("element", "No server found")

    def get_server_by_guid(
        self,
        server_guid: str = None,
        effective_time: str = None,
    ) -> str | dict:
        """Returns details about the platform's catalog entry (asset).

        Parameters
        ----------
        server_guid : str
            The unique identifier for the platform.
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_server_by_guid(server_guid, effective_time)
        )
        return response

    async def _async_get_servers_by_name(
        self,
        filter: str,
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of servers with a particular name.  The name is specified in the filter. Async version.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        url = (
            f"{self.runtime_command_root}/software-servers/by-name?"
            f"startFrom={start_from}&pageSize={page_size}"
        )

        if effective_time is None:
            body = {"filter": filter}
        else:
            body = {"filter": filter, "effective_time": effective_time}
        response = await self._async_make_request("POST", url, body)

        return response.json().get("elements", "No servers found")

    def get_servers_by_name(
        self, filter: str, effective_time: str = None
    ) -> str | list:
        """Returns the list of servers with a particular name.  The name is specified in the filter.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_servers_by_name(filter, effective_time)
        )
        return response

    async def _async_get_servers_by_dep_impl_type(
        self,
        filter: str = "*",
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of servers with a particular deployed implementation type. The value is specified
            in the filter. If it is null, or no request body is supplied, all servers are returned.
            Async version.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        if filter == "*":
            filter = None

        url = (
            f"{self.runtime_command_root}/software-servers/"
            f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}"
        )

        body = body_slimmer({"filter": filter, "effective_time": effective_time})

        response = await self._async_make_request("POST", url, body)

        return response.json().get("elements", "No servers found")

    def get_servers_by_dep_impl_type(
        self,
        filter: str = "*",
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of servers with a particular deployed implementation type.
            The value is specified in the filter. If it is null, or no request body is supplied,
            all servers are returned.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_servers_by_dep_impl_type(
                filter, effective_time, start_from, page_size
            )
        )
        return response

    async def _async_get_server_templates_by_dep_impl_type(
        self,
        filter: str = "*",
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of server templates with a particular deployed implementation type.   The value is
            specified in the filter. If it is null, or no request body is supplied, all servers are returned.
            Async version.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """

        if filter == "*":
            filter = None

        url = (
            f"{self.runtime_command_root}/software-servers/"
            f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}&getTemplates=true"
        )

        body = body_slimmer({"filter": filter, "effective_time": effective_time})

        response = await self._async_make_request("POST", url, body)

        return response.json().get("elements", "No platforms found")

    def get_server_templates_by_dep_impl_type(
        self,
        filter: str = "*",
        effective_time: str = None,
        start_from: int = 0,
        page_size: int = max_paging_size,
    ) -> str | list:
        """Returns the list of server templates with a particular deployed implementation type.
            The value is specified in the filter. If it is null, or no request body is supplied,
            all servers are returned.

        Parameters
        ----------
        filter : str, opt
            Filter specifies the kind of deployed implementation type of the platforms to return information for.
            If the value is None, we will default to the "OMAG Server Platform".
        effective_time: str, optional
            Timeframe to return information for. Time format is "YYYY-MM-DDTHH:MM:SS" (ISO 8601).
        start_from : int, optional
           The index from which to start fetching the engine actions. Default is 0.
        page_size : int, optional
           The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

        Returns
        -------
        Response
           A lit of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_server_templates_by_dep_impl_type(
                filter, effective_time, start_from, page_size
            )
        )
        return response

    async def _async_get_server_report(
        self, server_guid: str = None, server_name: str = None
    ) -> str | list:
        """Returns details about the running server. Async version.

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        server_name: str, default = None
            Name of server to act on. If not specified, server_guid must be.

        Returns
        -------
        Response
           A list of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        server_guid = self.__get_guid__(
            server_guid, server_name, "name", tech_type="Integration Daemon"
        )
        url = f"{self.runtime_command_root}/omag-servers/{server_guid}/instance/report"

        response = await self._async_make_request("GET", url)

        return response.json().get("element", "No server found")

    def get_server_report(
        self, server_guid: str = None, server_name: str = None
    ) -> str | list:
        """Returns details about the running server.

        Parameters
        ----------
        server_guid : str, default = None
            Identity of the server to act on. If not specified, server_name must be.
        server_name: str, default = None
            Name of server to act on. If not specified, server_guid must be.

        Returns
        -------
        Response
           A list of json dict with the platform reports.

        Raises
        ------
        InvalidParameterException
        PropertyServerException
        UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(
            self._async_get_server_report(server_guid, server_name)
        )
        return response


if __name__ == "__main__":
    print("Main-Runtime Manager")



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Display the server status of all registered servers across all platforms.

Note: This implementation is using the platform-report. A more complex platform environment may require
reimplementing from a server only perspective using get_servers_by_dep_impl_type() to find the guids and
then get_server_by_guid() to get the status. Less efficient but may be more robust for some environments.



A simple server status display
"""
import argparse
import os
import sys
import time

from rich.live import Live
from rich.table import Table

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    RuntimeManager,
    UserNotAuthorizedException,
    print_exception_response,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"
)
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))


def display_status(
    extended: bool,
    view_server: str = EGERIA_VIEW_SERVER,
    url: str = EGERIA_VIEW_SERVER_URL,
    username: str = EGERIA_USER,
    user_pass: str = EGERIA_USER_PASSWORD,
    jupyter: bool = EGERIA_JUPYTER,
    width: int = EGERIA_WIDTH,
):
    p_client = RuntimeManager(view_server, url, username, user_pass)
    token = p_client.create_egeria_bearer_token()

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Server Status for Platform - {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            caption=f"Server Status from Platform - '{url}'",
            show_lines=True,
        )

        table.add_column("Known Server")
        table.add_column("Status")
        if extended:
            table.add_column("GUID", no_wrap=True)
            table.add_column("Qualified Name")
            table.add_column("Description")
            table.add_column("Server Type")
            table.add_column("Last Started")

        platform_list = p_client.get_platforms_by_type()
        if type(platform_list) is str:
            print("No OMAG Server Platforms found?")
            sys.exit(1)

        for platform in platform_list:
            platform_name = platform["properties"].get("displayName", "---")
            platform_guid = platform["elementHeader"]["guid"]
            platform_desc = platform["properties"].get("resourceDescription", "---")
            p_server_list = ""

            platform_report = p_client.get_platform_report(platform_guid)
            servers = platform_report.get("omagservers", None)
            for p_server in servers:
                server_name = p_server.get("serverName", "--- ")
                server_type = p_server.get("serverType", "--- ")
                server_desc = p_server.get("description", "--- ")
                last_start_time = p_server.get("lastStartTime", "--- ")

                server_status = p_server.get("serverActiveStatus", "UNKNOWN")
                if server_status in ("RUNNING", "STARTING"):
                    status = "Active"
                elif server_status in ("INACTIVE", "STOPPING", "UNKNOWN"):
                    status = "Inactive"
                else:
                    status = "UNKNOWN"

                if extended:
                    #
                    #  get the qualified name and guid from get_server_by_name?
                    #
                    server_info = p_client.get_servers_by_name(server_name)
                    if type(server_info) is str:
                        guid = "---"
                        qualified_name = "---"
                    else:
                        guid = ""
                        qualified_name = ""
                        for i in server_info:
                            guid += f"{i['elementHeader']['guid']} "
                            qualified_name += f"{i['properties']['qualifiedName']} "

                    table.add_row(
                        server_name,
                        "[red]Inactive" if status == "Inactive" else "[green]Active",
                        guid,
                        qualified_name,
                        server_desc,
                        server_type,
                        last_start_time,
                    )
                else:
                    table.add_row(
                        server_name,
                        "[red]Inactive" "" if status == "Inactive" else "[green]Active",
                        # server_status,
                    )
        p_client.refresh_egeria_bearer_token()
        return table

    try:
        # with Live(generate_table(), refresh_per_second=1, screen=True) as live:
        print(f" Started at {time.asctime()} \n")
        with Live(
            generate_table(), refresh_per_second=4, screen=True, transient=True
        ) as live:

            while True:
                time.sleep(5)
                live.update(generate_table())

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        print_exception_response(e)
    except KeyboardInterrupt:
        pass
    finally:
        p_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--full", default=True, help="Extended Information?")
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    full = args.full if args.full is not None else False
    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    display_status(full, server, url, userid, user_pass)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A simple status display for the Integration Daemon.


"""
import argparse
import os
import time
from typing import Union

import nest_asyncio
from rich import box
from rich.console import Console
from rich.live import Live
from rich.prompt import Prompt
from rich.table import Table
from textual.widgets import DataTable

from pyegeria import AutomatedCuration, EgeriaTech
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON", "integration-daemon"
)
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"
)
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", 200))

disable_ssl_warnings = True


def display_integration_daemon_status(
    search_list: list[str] = ["*"],
    integ_server: str = os.environ.get(
        "EGERIA_INTEGRATION_DAEMON", "integration-daemon"
    ),
    integ_url: str = os.environ.get(
        "EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"
    ),
    view_server: str = os.environ.get("EGERIA_VIEW_SERVER", "view-server"),
    view_url: str = os.environ.get("EGERIA_VIEW_SERVER_URL", "https://localhost:9443"),
    user: str = os.environ.get("EGERIA_USER"),
    user_pass: str = os.environ.get("EGERIA_USER_PASSWORD"),
    paging: bool = False,
    jupyter: bool = os.environ.get("EGERIA_JUPYTER", "False"),
    width: int = EGERIA_WIDTH,
    sort: bool = True,
) -> None:
    """Display the status of connectors running on the specified Integration Daemon OMAG Server.

    Parameters
    ----------
    search_list : list[str], optional
        A list of connector names to search for. Default is ["*"], which returns all connectors.

    integ_server : str, optional
        The name of the integration daemon server. Default is EGERIA_INTEGRATION_DAEMON.

    integ_url : str, optional
        The URL of the integration daemon server. Default is EGERIA_INTEGRATION_DAEMON_URL.

    view_server : str, optional
        The name of the view server. Default is EGERIA_VIEW_SERVER.

    view_url : str, optional
        The URL of the view server. Default is EGERIA_VIEW_SERVER_URL.

    user : str, optional
        The username for authentication. Default is EGERIA_USER.

    user_pass : str, optional
        The password for authenticated access. Default is EGERIA_USER_PASSWORD.

    paging : bool, optional
        Determines whether to use paging or a live monitor for console output. Default is True.

    jupyter : bool, optional
        Determines whether running in a Jupyter environment. Default is EGERIA_JUPYTER.

    width : int, optional
        The width of the console display. Default is EGERIA_WIDTH.

    sort : bool, optional
        Determines whether to sort the connector reports. Default is True.

    Returns
    -------

    Nothing
    """
    s_client = EgeriaTech(view_server, view_url, user, user_pass)
    token = s_client.create_egeria_bearer_token()

    def generate_table(search_list: list[str]) -> Table:
        """Make a new table."""
        table = Table(
            title=f"Integration Daemon Status @ {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Integration Daemon Status for Server '{integ_server}' @ Platform - {integ_url}",
            expand=True,
        )
        table.add_column("Connector Name", min_width=15)
        table.add_column("Status", max_width=6)

        table.add_column("Last Refresh Time", min_width=12)
        table.add_column("Min Refresh (mins)", max_width=6)
        table.add_column("Target Element", min_width=20)
        table.add_column("Exception Message", min_width=10)

        # server_guid = s_client.get_guid_for_name(integ_server)

        token = s_client.refresh_egeria_bearer_token()
        daemon_status = s_client.get_server_report(None, integ_server)

        reports = daemon_status["integrationConnectorReports"]
        if sort is True:
            connector_reports = sorted(
                reports, key=lambda x: x.get("connectorName", "---")
            )
        else:
            connector_reports = reports

        for connector in connector_reports:
            connector_name = connector.get("connectorName", "---")

            if (connector_name not in search_list) and (search_list != ["*"]):
                # if specific connectors are requested and it doesn't match, then skip
                continue

            connector_status = connector.get("connectorStatus", "---")
            connector_guid = connector.get("connectorGUID", "---")
            last_refresh_time = connector.get("lastRefreshTime", "---")[:-10]
            refresh_interval = str(connector.get("minMinutesBetweenRefresh", "---"))
            exception_msg = connector.get("failingExceptionMessage", " ")
            if connector_guid != "---":
                targets = s_client.get_catalog_targets(connector_guid)
                tgt_tab = Table()
                tgt_tab.add_column("Target")
                tgt_tab.add_column("UniqueName")
                tgt_tab.add_column("Relationship GUID", no_wrap=True)

                if type(targets) == list:
                    targets_md = True
                    for target in targets:
                        t_name = target["catalogTargetName"]
                        # t_sync = target["permittedSynchronization"]
                        t_unique_name = target["catalogTargetElement"]["uniqueName"]
                        t_rel_guid = target["relationshipGUID"]
                        # targets_m += f"* Target Name: __{t_name}__\n* Sync: {t_sync}\n* Unique Name: {t_unique_name}\n\n"
                        tgt_tab.add_row(t_name, t_unique_name, t_rel_guid)
                    # targets_md = Markdown(targets_m)
                else:
                    targets_md = False
            else:
                targets_md = False
            if targets_md is False:
                targets_out = ""
            else:
                targets_out = tgt_tab

            if connector_status in ("RUNNING", "REFRESHING", "WAITING"):
                connector_status = f"[green]{connector_status}"
            elif connector_status in ("INITIALIZE FAILED", "CONFIG_FAILED", "FAILED"):
                connector_status = f"[red]{connector_status}"
            else:
                connector_status = f"[yellow]{connector_status}"

            table.add_row(
                connector_name,
                connector_status,
                last_refresh_time,
                refresh_interval,
                targets_out,
                exception_msg,
            )

        return table

    try:
        console = Console(width=width)  # main_pagig, force_terminal=not jupyter)
        if paging is True:
            with console.pager():
                console.print(generate_table(search_list))
        else:
            with Live(
                generate_table(search_list),
                refresh_per_second=1,
                screen=True,
                vertical_overflow="visible",
            ) as live:
                while True:
                    time.sleep(2)
                    live.update(generate_table(search_list))

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        print_exception_response(e)

    except KeyboardInterrupt:
        pass

    finally:
        s_client.close_session()


def main_live(paging: bool = False) -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--integ_server", help="Name of the integration server to display status for"
    )
    parser.add_argument("--integ_url", help="URL Platform to connect to")
    parser.add_argument("--view_server", help="Name of the view server to use")
    parser.add_argument("--view_url", help="view server URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    integ_server = (
        args.integ_server
        if args.integ_server is not None
        else EGERIA_INTEGRATION_DAEMON
    )
    integ_url = (
        args.integ_url if args.integ_url is not None else EGERIA_INTEGRATION_DAEMON_URL
    )
    view_server = (
        args.view_server if args.view_server is not None else EGERIA_VIEW_SERVER
    )
    view_url = args.view_url if args.view_url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    search_list = Prompt.ask(
        "Enter the list of connectors you are interested in or '*' for all",
        default="*",
    )
    display_integration_daemon_status(
        search_list=[search_list],
        integ_server=integ_server,
        integ_url=integ_url,
        view_server=view_server,
        view_url=view_url,
        user=userid,
        user_pass=user_pass,
        paging=paging,
    )


def main_paging(paging: bool = True) -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--integ_server", help="Name of the integration server to display status for"
    )
    parser.add_argument("--integ_url", help="URL Platform to connect to")
    parser.add_argument("--view_server", help="Name of the view server to use")
    parser.add_argument("--view_url", help="view server URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    integ_server = (
        args.integ_server
        if args.integ_server is not None
        else EGERIA_INTEGRATION_DAEMON
    )
    integ_url = (
        args.integ_url if args.integ_url is not None else EGERIA_INTEGRATION_DAEMON_URL
    )
    view_server = (
        args.view_server if args.view_server is not None else EGERIA_VIEW_SERVER
    )
    view_url = args.view_url if args.view_url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    search_list = Prompt.ask(
        "Enter the list of connectors you are interested in or '*' for all",
        default="*",
    )
    display_integration_daemon_status(
        search_list=[search_list],
        integ_server=integ_server,
        integ_url=integ_url,
        view_server=view_server,
        view_url=view_url,
        user=userid,
        user_pass=user_pass,
        paging=paging,
    )


if __name__ == "__main__":
    main_live(paging=False)

if __name__ == "__main_paging__":
    main_paging(paging=True)



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module provides a set of utility functions to render Mermaid markdown in a Jupyter notebook.

A running Egeria environment is needed to run these functions.
These functions have been tested in a Jupyter notebook - but may work in other environments.

"""
import html
import os
import time
import uuid

import nest_asyncio

nest_asyncio.apply()
from IPython.display import HTML, display
from rich.console import Console

from pyegeria.automated_curation_omvs import AutomatedCuration
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
)
from pyegeria._globals import NO_ELEMENTS_FOUND

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("INTEGRATION_DAEMON", "integration-daemon")
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))
EGERIA_MERMAID_FOLDER = os.environ.get("EGERIA_MERMAID_FOLDER", "./work/mermaid_graphs")


def load_mermaid():
    """Inject Mermaid.js library"""
    # Alternative CDN URL via unpkg
    mermaid_js = """
    <script src="https://unpkg.com/mermaid@11.6.0/dist/mermaid.min.js"></script>
    <script>
        document.addEventListener('DOMContentLoaded', function() {
            mermaid.initialize({startOnLoad: true},
            {maxTextSize: 300000});
        });
    </script>

    """

    display(HTML(mermaid_js))


def render_mermaid(mermaid_code):
    title_label, guid, mermaid_code = parse_mermaid_code(mermaid_code)
    graph_id = f"mermaid-graph-{guid}-{str(uuid.uuid4())[:4]}"
    escaped_header = html.escape(title_label) if title_label else ""  # Sanitize the header safely
    escaped_mermaid_code = html.escape(mermaid_code)
    header_html = f"""
                <h3 id="{graph_id}-heading" style="margin: 20px 0; font-size: 1.5em; text-align: center;">
                    {escaped_header}
                </h3>
                <p id="{graph_id}-subheading" style="margin: 0; padding: 5px; font-size: 1em; text-align: center; color: gray; flex: 0 0 auto;">
                    GUID: {guid}
                </p>
                """ if title_label else ""

    mermaid_html = f"""
    <div>{header_html}</div>
    <div class="mermaid">
        {escaped_mermaid_code}
    </div>
    <script type="text/javascript">
        if (window.mermaid) {{
    mermaid.initialize({{startOnLoad: true}});
    mermaid.contentLoaded();
    }}
    </script>
    """
    display(HTML(mermaid_html))


def render_mermaid_adv(mermaid_code):
    return display(HTML(construct_mermaid_web(mermaid_code)),)

def parse_mermaid_code(mermaid_code):
    parts = mermaid_code.split("---", maxsplit=3)
    guid = None
    if len(parts) == 3:
        full_title = parts[1].strip()
        if "[" in full_title:
            title_l = full_title.split("[")[0]
            title = title_l.replace("title: ", "")
            guid = full_title.split("[")[1].split("]")[0]
        else:
            title = full_title.replace("title: ", "")
        mermaid_code = parts[2].strip()
    else:
        title = None
    if guid is None:
        guid = str(uuid.uuid4())[:8]
    return title, guid, mermaid_code


def construct_mermaid_web(mermaid_str: str) -> str:
    """Function to display a HTML code in a Jupyter notebook"""
    title_label, guid, mermaid_code = parse_mermaid_code(mermaid_str)

    html_section1 = """
    <!DOCTYPE html>
    <html>
        <head>
          <style type="text/css">
            #mySvgId {
            width: 100%;
            height: 1200px;
            overflow: scroll;
            border: 2px solid #ccc;
            position: relative;
            margin-bottom: 10px;
            }
            svg {
            cursor: grab;
            }
    
          </style>
        </head>
    """
    html_section2 = f"""
        <title>{title_label}</title>
        <h3>{title_label}</h3>
        GUID : {guid}

    """
    html_section3 = """
        <body>
        
          <div id="graphDiv"></div>
          <script src="https://bumbu.me/svg-pan-zoom/dist/svg-pan-zoom.min.js"></script>
          <script type="module">
            import mermaid from 'https://cdn.jsdelivr.net/npm/mermaid@11/dist/mermaid.esm.min.mjs';
    
            mermaid.initialize({startOnLoad: false});
            await mermaid.run({
              querySelector: '.mermaid',
              postRenderCallback: (id) => {
                const container = document.getElementById("diagram-container");
                const svgElement = container.querySelector("svg");
    
                // Initialize Panzoom
                const panzoomInstance = Panzoom(svgElement, {
                  maxScale: 5,
                  minScale: 0.5,
                  step: 0.1,
                });
    
                // Add mouse wheel zoom
                container.addEventListener("wheel", (event) => {
                  panzoomInstance.zoomWithWheel(event);
                });
              }
            });
    
    
            const drawDiagram = async function () {
              const element = document.querySelector('#graphDiv');
             const graphDefinition = `
    """

    html_section4 = r"""`;
              const {svg} = await mermaid.render('mySvgId', graphDefinition);
              element.innerHTML = svg.replace(/( )*max-width:( 0-9\.)*px;/i, '');
    
              var doPan = false;
              var eventsHandler;
              var panZoom;
              var mousepos;
    
              eventsHandler = {
                haltEventListeners: ['mousedown', 'mousemove', 'mouseup']
    
                , mouseDownHandler: function (ev) {
                  if (event.target.className == "[object SVGAnimatedString]") {
                    doPan = true;
                    mousepos = {x: ev.clientX, y: ev.clientY}
                  }
                  ;
                }
    
                , mouseMoveHandler: function (ev) {
                  if (doPan) {
                    panZoom.panBy({x: ev.clientX - mousepos.x, y: ev.clientY - mousepos.y});
                    mousepos = {x: ev.clientX, y: ev.clientY};
                    window.getSelection().removeAllRanges();
                  }
                }
    
                , mouseUpHandler: function (ev) {
                  doPan = false;
                }
    
                , init: function (options) {
                  options.svgElement.addEventListener('mousedown', this.mouseDownHandler, false);
                  options.svgElement.addEventListener('mousemove', this.mouseMoveHandler, false);
                  options.svgElement.addEventListener('mouseup', this.mouseUpHandler, false);
                }
    
                , destroy: function (options) {
                  options.svgElement.removeEventListener('mousedown', this.mouseDownHandler, false);
                  options.svgElement.removeEventListener('mousemove', this.mouseMoveHandler, false);
                  options.svgElement.removeEventListener('mouseup', this.mouseUpHandler, false);
                }
              }
              panZoom = svgPanZoom('#mySvgId', {
                zoomEnabled: true
                , controlIconsEnabled: true
                , fit: 1
                , center: 1
                , customEventsHandler: eventsHandler
              })
            };
            await drawDiagram();
          </script>
        </body>    
    """

    return html_section1 + html_section2 + html_section3 + mermaid_code + html_section4

def construct_mermaid_jup(mermaid_str: str) -> str:
    """Function to display a HTML code in a Jupyter notebook

    Constructs HTML for a single Mermaid graph with pan and zoom support.
    Each call overwrites the previous graph.

    :param mermaid_code: The Mermaid code for the graph.
    :param graph_id: An optional unique graph ID (default is 'mermaid-graph').
    :return: The HTML content for the Mermaid graph with pan and zoom enabled.

    """
    title_label, guid, mermaid_code = parse_mermaid_code(mermaid_str)

    graph_id = f"mermaid-graph-{guid}-{str(uuid.uuid4())[:4]}"
    escaped_header = html.escape(title_label) if title_label else ""  # Sanitize the header safely
    escaped_mermaid_code = html.escape(mermaid_code)
    header_html = f"""
                <h3 id="{graph_id}-heading" style="margin: 20px 0; font-size: 1.5em; text-align: center;">
                    {escaped_header}
                </h3>
                <p id="{graph_id}-subheading" style="margin: 0; padding: 5px; font-size: 1em; text-align: center; color: gray; flex: 0 0 auto;">
                    GUID: {guid}
                </p>
                """ if title_label else ""

    html_content = f"""
            <style>
        /* Style for the diagram container */
        .diagram-container {{
            position: relative;
            width: 100%; /* Adjust the diagram container width */
            height: 500px; /* Set a fixed height for the container */
            margin: 0 auto;
            border: 1px solid #ccc; /* Optional border for visualization */
            overflow: hidden; /* Prevent content overflow outside the container */
        }}

        /* Style for zoom controls */
        .svg-pan-zoom_controls {{
            position: absolute;
            top: 10px;
            right: 10px;
            display: flex;
            flex-direction: column;
            gap: 5px;
        }}
        .svg-pan-zoom_controls button {{
            background-color: #007bff;
            color: white;
            border: none;
            padding: 5px 10px;
            border-radius: 3px;
            cursor: pointer;
            font-size: 14px;
        }}
        .svg-pan-zoom_controls button:hover {{
            background-color: #0056b3;
        }}
    </style>

    <div id="{graph_id}-container" class="diagram-container">
        <!-- Mermaid diagram will be dynamically rendered here -->
        {header_html}
        <div id="{graph_id}" class="mermaid">
            {escaped_mermaid_code}
        </div>
    </div>

    <script src="https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js"></script>
        <script src="https://cdn.jsdelivr.net/npm/svg-pan-zoom@3.6.1/dist/svg-pan-zoom.min.js"></script>
        <script>
            document.addEventListener("DOMContentLoaded", () => {{
                // Initialize Mermaid and Pan-Zoom functionality
                const graph_id = "{graph_id}";

                function initializeMermaid(graph_id) {{
                    const containerElement = document.getElementById(`${{graph_id}}-container`);

                    if (!containerElement) {{
                        console.error(`Container with ID "${{graph_id}}-container" not found.`);
                        return;
                    }}

                    // Configure Mermaid
                    mermaid.initialize({{ startOnLoad: false, logLevel: "debug" }});
                    mermaid.init(undefined, `#${{graph_id}}`);

                    setTimeout(() => {{
                        const svg = containerElement.querySelector("svg");
                        if (!svg) {{
                            console.error(`SVG not rendered for ID "${{graph_id}}".`);
                            return;
                        }}

                        // Set initial size
                        svg.setAttribute("width", "100%");
                        svg.setAttribute("height", "100%");

                        // Initialize Pan-Zoom
                        const panZoom = svgPanZoom(svg, {{
                            zoomEnabled: true,
                            controlIconsEnabled: false,
                            fit: true,
                            center: true,
                            minZoom: 0.5,
                            maxZoom: 10,
                            contain: true
                        }});

                        // Add custom controls
                        const controlsContainer = document.createElement("div");
                        controlsContainer.className = "svg-pan-zoom_controls";
                        controlsContainer.innerHTML = `
                            <button id="${{graph_id}}-zoom-in">+</button>
                            <button id="${{graph_id}}-zoom-out">-</button>
                            <button id="${{graph_id}}-reset">Reset</button>
                        `;
                        containerElement.appendChild(controlsContainer);

                        // Handle controls
                        document.getElementById(`${{graph_id}}-zoom-in`).addEventListener("click", () => panZoom.zoomIn());
                        document.getElementById(`${{graph_id}}-zoom-out`).addEventListener("click", () => panZoom.zoomOut());
                        document.getElementById(`${{graph_id}}-reset`).addEventListener("click", () => {{
                            panZoom.resetZoom();
                            panZoom.center();
                        }});
                    }}, 500);
                }}

                if (typeof mermaid === "undefined") {{
                    const script = document.createElement('script');
                    script.src = "https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js";
                    script.onload = () => initializeMermaid(graph_id);
                    document.head.appendChild(script);
                }} else {{
                    initializeMermaid(graph_id);
                }}
            }});
        </script>

    """
    return html_content




def not_working_construct_mermaid_html(mermaid_str: str) -> str:
    """Function to display a HTML code in a Jupyter notebook

    Constructs HTML for a single Mermaid graph with pan and zoom support.
    Each call overwrites the previous graph.

    :param mermaid_code: The Mermaid code for the graph.
    :param graph_id: An optional unique graph ID (default is 'mermaid-graph').
    :return: The HTML content for the Mermaid graph with pan and zoom enabled.

    """
    title_label, guid, mermaid_code = parse_mermaid_code(mermaid_str)

    graph_id = f"mermaid-graph-{guid}-{str(uuid.uuid4())[:4]}"
    escaped_header = html.escape(title_label) if title_label else ""  # Sanitize the header safely
    escaped_mermaid_code = html.escape(mermaid_code)
    header_html = f"""
            <h3 id="{graph_id}-heading" style="margin: 20px 0; font-size: 1.5em; text-align: center;">
                {escaped_header}
            </h3>
            <p id="{graph_id}-subheading" style="margin: 0; padding: 5px; font-size: 1em; text-align: center; color: gray; flex: 0 0 auto;">
                GUID: {guid}
            </p>
            """ if title_label else ""

    html_content = f"""
        <style>
    /* Style for the diagram container */
    .diagram-container {{
        position: relative;
        width: 100%; /* Adjust the diagram container width */
        height: 500px; /* Set a fixed height for the container */
        margin: 0 auto;
        border: 1px solid #ccc; /* Optional border for visualization */
        overflow: hidden; /* Prevent content overflow outside the container */
    }}

    /* Style for zoom controls */
    .svg-pan-zoom_controls {{
        position: absolute;
        top: 10px;
        right: 10px;
        display: flex;
        flex-direction: column;
        gap: 5px;
    }}
    .svg-pan-zoom_controls button {{
        background-color: #007bff;
        color: white;
        border: none;
        padding: 5px 10px;
        border-radius: 3px;
        cursor: pointer;
        font-size: 14px;
    }}
    .svg-pan-zoom_controls button:hover {{
        background-color: #0056b3;
    }}
</style>

<div id="{graph_id}-container" class="diagram-container">
    <!-- Mermaid diagram will be dynamically rendered here -->
    {header_html}
    <div id="{graph_id}" class="mermaid">
        {escaped_mermaid_code}
    </div>
</div>

<script src="https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js"></script>
    <script src="https://cdn.jsdelivr.net/npm/svg-pan-zoom@3.6.1/dist/svg-pan-zoom.min.js"></script>
    <script>
        document.addEventListener("DOMContentLoaded", () => {{
            // Initialize Mermaid and Pan-Zoom functionality
            const graph_id = "{graph_id}";

            function initializeMermaid(graph_id) {{
                const containerElement = document.getElementById(`${{graph_id}}-container`);

                if (!containerElement) {{
                    console.error(`Container with ID "${{graph_id}}-container" not found.`);
                    return;
                }}

                // Configure Mermaid
                mermaid.initialize({{ startOnLoad: false, logLevel: "debug" }});
                mermaid.init(undefined, `#${{graph_id}}`);

                setTimeout(() => {{
                    const svg = containerElement.querySelector("svg");
                    if (!svg) {{
                        console.error(`SVG not rendered for ID "${{graph_id}}".`);
                        return;
                    }}

                    // Set initial size
                    svg.setAttribute("width", "100%");
                    svg.setAttribute("height", "100%");

                    // Initialize Pan-Zoom
                    const panZoom = svgPanZoom(svg, {{
                        zoomEnabled: true,
                        controlIconsEnabled: false,
                        fit: true,
                        center: true,
                        minZoom: 0.5,
                        maxZoom: 10,
                        contain: true
                    }});

                    // Add custom controls
                    const controlsContainer = document.createElement("div");
                    controlsContainer.className = "svg-pan-zoom_controls";
                    controlsContainer.innerHTML = `
                        <button id="${{graph_id}}-zoom-in">+</button>
                        <button id="${{graph_id}}-zoom-out">-</button>
                        <button id="${{graph_id}}-reset">Reset</button>
                    `;
                    containerElement.appendChild(controlsContainer);

                    // Handle controls
                    document.getElementById(`${{graph_id}}-zoom-in`).addEventListener("click", () => panZoom.zoomIn());
                    document.getElementById(`${{graph_id}}-zoom-out`).addEventListener("click", () => panZoom.zoomOut());
                    document.getElementById(`${{graph_id}}-reset`).addEventListener("click", () => {{
                        panZoom.resetZoom();
                        panZoom.center();
                    }});
                }}, 500);
            }}

            if (typeof mermaid === "undefined") {{
                const script = document.createElement('script');
                script.src = "https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js";
                script.onload = () => initializeMermaid(graph_id);
                document.head.appendChild(script);
            }} else {{
                initializeMermaid(graph_id);
            }}
        }});
    </script>

"""
#
#     return html_content
# This one almost works - no controls and diagram too small
    # html_content = f"""
# <!DOCTYPE html>
# <html>
# <head>
#     <style>
#         /* Set consistent sizing and prevent shrinking for the container and parents */
#         html, body, #{graph_id}-wrapper {{
#             margin: 0;
#             padding: 0;
#             height: 100%; /* Ensure the entire root hierarchy respects 100% height */
#             width: 100%;
#         }}
#
#         /* Ensure diagram container maintains full height and prevents overflow */
#         #{graph_id}-container {{
#             position: relative;
#             width: 100%;
#             height: 100%;
#             overflow: hidden; /* Prevent content from breaking layout */
#             display: flex;
#             align-items: center; /* Centers the diagram vertically */
#             justify-content: center; /* Centers the diagram horizontally */
#         }}
#
#         /* Ensure SVG always stretches to match container */
#         #{graph_id} svg {{
#             width: 100%;
#             height: 100%;
#             display: block; /* Avoid unwanted inline space issues */
#         }}
#
#         /* Optional custom pan/zoom controls (if dynamically added) */
#         .svg-pan-zoom_controls {{
#             position: absolute;
#             bottom: 10px; /* Adjust based on desired placement */
#             right: 10px;
#             z-index: 10;
#             display: flex;
#             gap: 5px;
#         }}
#
#         .svg-pan-zoom_controls button {{
#             border: 1px solid #ccc;
#             background-color: #fff;
#             color: #000;
#             padding: 5px 10px;
#             cursor: pointer;
#             font-size: 16px;
#         }}
#
#         .svg-pan-zoom_controls button:hover {{
#             background-color: #f0f0f0;
#         }}
#     </style>
#     </head>
#     <body>
#     <div id="{graph_id}-wrapper">
#         <!-- Title/Heading (optional; include as needed) -->
#         {escaped_header if escaped_header else ""}
#
#         <!-- Mermaid Diagram Container -->
#         <div id="{graph_id}-container">
#             <div id="{graph_id}" class="mermaid">
#                 {escaped_mermaid_code}
#             </div>
#         </div>
#     </div>
#
#     <script src="https://cdn.jsdelivr.net/npm/svg-pan-zoom@3.6.1/dist/svg-pan-zoom.min.js"></script>
#     <script>
#         document.addEventListener("DOMContentLoaded", () => {{
#                                                              // Define the graph ID dynamically passed via Python
#         const graph_id = "{graph_id}";
#
#         // Function to load external scripts dynamically
#         function loadScript(url, callback) {{
#             const script = document.createElement('script');
#         script.src = url;
#         script.async = true;
#         script.onload = callback;
#         script.onerror = () => console.error("Error loading script:", url);
#         document.head.appendChild(script);
#         }}
#
#         // Function to initialize Mermaid
#         function initializeMermaid(graph_id) {{
#                                               // Look for the container ID generated dynamically by the template
#         const containerElement = document.getElementById(`${{graph_id}}-container`);
#         if (!containerElement) {{
#         console.error(`Container element with id "${{graph_id}}-container" not found.`);
#         return;
#         }}
#
#         // Initialize Mermaid.js with default configuration
#         mermaid.initialize({{ startOnLoad: false, logLevel: "debug" }});
#         mermaid.init(undefined, `#${{graph_id}}`); // Initialize element with id matching `graph_id`
#
#                      // Add a timeout to ensure the SVG is rendered
#         setTimeout(() => {{
#             const svg = containerElement.querySelector("svg");
#
#         // Handle errors if SVG rendering fails
#         if (!svg) {{
#             console.error(`SVG not rendered for ID "${{graph_id}}". Check Mermaid syntax or rendering issues.`);
#         console.log("Container content:", containerElement.innerHTML); // Log the container's content
#         return;
#         }}
#
#         // Set SVG attributes for full container fit
#         svg.setAttribute("width", "100%");
#         svg.setAttribute("height", "100%");
#
#         // Initialize SVG Pan-Zoom functionality
#         const panZoom = svgPanZoom(svg, {{
#         zoomEnabled: true,
#         controlIconsEnabled: true, // Display default controls if enabled
#         fit: true,  // Fit diagram within the container
#         center: true, // Center the diagram
#         minZoom: 0.5, // Prevent too much zooming out
#         maxZoom: 10,  // Prevent extreme zooming in
#                                          contain: true // Keep the diagram fully within bounds
#         }});
#
#         // Optional: Add custom pan/zoom controls dynamically
#             const controlsContainer = document.createElement("div");
#             controlsContainer.classList.add("svg-pan-zoom_controls");
#             controlsContainer.innerHTML = `
#                                         <button id="${{graph_id}}-zoom-in">+</button>
#                                         <button id="${{graph_id}}-zoom-out">-</button>
#                                         <button id="${{graph_id}}-reset">Reset</button>
#                                          `;
#             containerElement.appendChild(controlsContainer);
#
#         // Add event listeners for the custom controls
#         document.getElementById(`${{graph_id}}-zoom-in`).addEventListener("click", () => panZoom.zoomIn());
#         document.getElementById(`${{graph_id}}-zoom-out`).addEventListener("click", () => panZoom.zoomOut());
#         document.getElementById(`${{graph_id}}-reset`).addEventListener("click", () => {{
#                 panZoom.resetZoom();
#                 panZoom.center();
#                 }});
#             }}, 500); // Short delay to ensure SVG rendering is complete
#         }}
#
#         // Load Mermaid.js if not already loaded
#         if (typeof mermaid === "undefined") {{
#             loadScript("https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js", () => {{
#             console.log("Mermaid.js loaded successfully");
#             initializeMermaid(graph_id); // Call initialization after Mermaid.js is loaded
#         }});
#             }} else {{
#                 initializeMermaid(graph_id); // If Mermaid.js is already loaded, proceed immediately
#             }}
#         }});
#     </script>
#
#
#     </body>
#     </html>
#     """





    return html_content

    # // Check if Mermaid and SVG-Pan-Zoom are loaded
    # const loadMermaid = typeof mermaid === "undefined";
    # const loadSvgPanZoom = typeof svgPanZoom === "undefined";
    #
    # if (loadMermaid || loadSvgPanZoom) {{
    #     const promises = [];
    #
    #     // Load Mermaid.js if not already loaded
    #     if (loadMermaid) {{
    #         promises.push(new Promise((resolve) => {{
    #             const script = document.createElement("script");
    #             script.src = "https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js";
    #             script.onload = resolve;
    #             document.head.appendChild(script);
    #         }}));
    #     }}
    #
    #     // Load SVG-Pan-Zoom if not already loaded
    #     if (loadSvgPanZoom) {{
    #         promises.push(new Promise((resolve) => {{
    #             const script = document.createElement("script");
    #             script.src = "https://cdn.jsdelivr.net/npm/svg-pan-zoom@3.6.1/dist/svg-pan-zoom.min.js";
    #             script.onload = resolve;
    #             document.head.appendChild(script);
    #         }}));
    #     }}
    #
    #     Promise.all(promises).then(() => initializeMermaid(graph_id));
    # }} else {{
    #     initializeMermaid(graph_id);
    # }}
    # }})();


#     html_content = f"""
# <style>
#     /* Style for the diagram container */
#     .diagram-container {{
#         position: relative;
#         width: 100%; /* Adjust the diagram container width */
#         height: 500px; /* Set a fixed height for the container */
#         margin: 0 auto;
#         border: 1px solid #ccc; /* Optional border for visualization */
#         overflow: hidden; /* Prevent content overflow outside the container */
#     }}
#
#     /* Style for zoom controls */
#     .svg-pan-zoom_controls {{
#         position: absolute;
#         top: 10px;
#         right: 10px;
#         display: flex;
#         flex-direction: column;
#         gap: 5px;
#     }}
#     .svg-pan-zoom_controls button {{
#         background-color: #007bff;
#         color: white;
#         border: none;
#         padding: 5px 10px;
#         border-radius: 3px;
#         cursor: pointer;
#         font-size: 14px;
#     }}
#     .svg-pan-zoom_controls button:hover {{
#         background-color: #0056b3;
#     }}
# </style>
#
# <div id="{graph_id}-container" class="diagram-container">
#     <!-- Mermaid diagram will be dynamically rendered here -->
#     <div id="{graph_id}" class="mermaid">
#         {mermaid_code}
#     </div>
# </div>
#
# <script src="https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js"></script>
#     <script src="https://cdn.jsdelivr.net/npm/svg-pan-zoom@3.6.1/dist/svg-pan-zoom.min.js"></script>
#     <script>
#         document.addEventListener("DOMContentLoaded", () => {{
#             // Initialize Mermaid and Pan-Zoom functionality
#             const graph_id = "{graph_id}";
#
#             function initializeMermaid(graph_id) {{
#                 const containerElement = document.getElementById(`${{graph_id}}-container`);
#
#                 if (!containerElement) {{
#                     console.error(`Container with ID "${{graph_id}}-container" not found.`);
#                     return;
#                 }}
#
#                 // Configure Mermaid
#                 mermaid.initialize({{ startOnLoad: false, logLevel: "debug" }});
#                 mermaid.init(undefined, `#${{graph_id}}`);
#
#                 setTimeout(() => {{
#                     const svg = containerElement.querySelector("svg");
#                     if (!svg) {{
#                         console.error(`SVG not rendered for ID "${{graph_id}}".`);
#                         return;
#                     }}
#
#                     // Set initial size
#                     svg.setAttribute("width", "100%");
#                     svg.setAttribute("height", "100%");
#
#                     // Initialize Pan-Zoom
#                     const panZoom = svgPanZoom(svg, {{
#                         zoomEnabled: true,
#                         controlIconsEnabled: false,
#                         fit: true,
#                         center: true,
#                         minZoom: 0.5,
#                         maxZoom: 10,
#                         contain: true
#                     }});
#
#                     // Add custom controls
#                     const controlsContainer = document.createElement("div");
#                     controlsContainer.className = "svg-pan-zoom_controls";
#                     controlsContainer.innerHTML = `
#                         <button id="${{graph_id}}-zoom-in">+</button>
#                         <button id="${{graph_id}}-zoom-out">-</button>
#                         <button id="${{graph_id}}-reset">Reset</button>
#                     `;
#                     containerElement.appendChild(controlsContainer);
#
#                     // Handle controls
#                     document.getElementById(`${{graph_id}}-zoom-in`).addEventListener("click", () => panZoom.zoomIn());
#                     document.getElementById(`${{graph_id}}-zoom-out`).addEventListener("click", () => panZoom.zoomOut());
#                     document.getElementById(`${{graph_id}}-reset`).addEventListener("click", () => {{
#                         panZoom.resetZoom();
#                         panZoom.center();
#                     }});
#                 }}, 500);
#             }}
#
#             if (typeof mermaid === "undefined") {{
#                 const script = document.createElement('script');
#                 script.src = "https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js";
#                 script.onload = () => initializeMermaid(graph_id);
#                 document.head.appendChild(script);
#             }} else {{
#                 initializeMermaid(graph_id);
#             }}
#         }});
#     </script>
#
# """
#
#     return html_content
# This one almost works - no controls and diagram too small
    # html_content = f"""
# <!DOCTYPE html>
# <html>
# <head>
#     <style>
#         /* Set consistent sizing and prevent shrinking for the container and parents */
#         html, body, #{graph_id}-wrapper {{
#             margin: 0;
#             padding: 0;
#             height: 100%; /* Ensure the entire root hierarchy respects 100% height */
#             width: 100%;
#         }}
#
#         /* Ensure diagram container maintains full height and prevents overflow */
#         #{graph_id}-container {{
#             position: relative;
#             width: 100%;
#             height: 100%;
#             overflow: hidden; /* Prevent content from breaking layout */
#             display: flex;
#             align-items: center; /* Centers the diagram vertically */
#             justify-content: center; /* Centers the diagram horizontally */
#         }}
#
#         /* Ensure SVG always stretches to match container */
#         #{graph_id} svg {{
#             width: 100%;
#             height: 100%;
#             display: block; /* Avoid unwanted inline space issues */
#         }}
#
#         /* Optional custom pan/zoom controls (if dynamically added) */
#         .svg-pan-zoom_controls {{
#             position: absolute;
#             bottom: 10px; /* Adjust based on desired placement */
#             right: 10px;
#             z-index: 10;
#             display: flex;
#             gap: 5px;
#         }}
#
#         .svg-pan-zoom_controls button {{
#             border: 1px solid #ccc;
#             background-color: #fff;
#             color: #000;
#             padding: 5px 10px;
#             cursor: pointer;
#             font-size: 16px;
#         }}
#
#         .svg-pan-zoom_controls button:hover {{
#             background-color: #f0f0f0;
#         }}
#     </style>
#     </head>
#     <body>
#     <div id="{graph_id}-wrapper">
#         <!-- Title/Heading (optional; include as needed) -->
#         {escaped_header if escaped_header else ""}
#
#         <!-- Mermaid Diagram Container -->
#         <div id="{graph_id}-container">
#             <div id="{graph_id}" class="mermaid">
#                 {escaped_mermaid_code}
#             </div>
#         </div>
#     </div>
#
#     <script src="https://cdn.jsdelivr.net/npm/svg-pan-zoom@3.6.1/dist/svg-pan-zoom.min.js"></script>
#     <script>
#         document.addEventListener("DOMContentLoaded", () => {{
#                                                              // Define the graph ID dynamically passed via Python
#         const graph_id = "{graph_id}";
#
#         // Function to load external scripts dynamically
#         function loadScript(url, callback) {{
#             const script = document.createElement('script');
#         script.src = url;
#         script.async = true;
#         script.onload = callback;
#         script.onerror = () => console.error("Error loading script:", url);
#         document.head.appendChild(script);
#         }}
#
#         // Function to initialize Mermaid
#         function initializeMermaid(graph_id) {{
#                                               // Look for the container ID generated dynamically by the template
#         const containerElement = document.getElementById(`${{graph_id}}-container`);
#         if (!containerElement) {{
#         console.error(`Container element with id "${{graph_id}}-container" not found.`);
#         return;
#         }}
#
#         // Initialize Mermaid.js with default configuration
#         mermaid.initialize({{ startOnLoad: false, logLevel: "debug" }});
#         mermaid.init(undefined, `#${{graph_id}}`); // Initialize element with id matching `graph_id`
#
#                      // Add a timeout to ensure the SVG is rendered
#         setTimeout(() => {{
#             const svg = containerElement.querySelector("svg");
#
#         // Handle errors if SVG rendering fails
#         if (!svg) {{
#             console.error(`SVG not rendered for ID "${{graph_id}}". Check Mermaid syntax or rendering issues.`);
#         console.log("Container content:", containerElement.innerHTML); // Log the container's content
#         return;
#         }}
#
#         // Set SVG attributes for full container fit
#         svg.setAttribute("width", "100%");
#         svg.setAttribute("height", "100%");
#
#         // Initialize SVG Pan-Zoom functionality
#         const panZoom = svgPanZoom(svg, {{
#         zoomEnabled: true,
#         controlIconsEnabled: true, // Display default controls if enabled
#         fit: true,  // Fit diagram within the container
#         center: true, // Center the diagram
#         minZoom: 0.5, // Prevent too much zooming out
#         maxZoom: 10,  // Prevent extreme zooming in
#                                          contain: true // Keep the diagram fully within bounds
#         }});
#
#         // Optional: Add custom pan/zoom controls dynamically
#             const controlsContainer = document.createElement("div");
#             controlsContainer.classList.add("svg-pan-zoom_controls");
#             controlsContainer.innerHTML = `
#                                         <button id="${{graph_id}}-zoom-in">+</button>
#                                         <button id="${{graph_id}}-zoom-out">-</button>
#                                         <button id="${{graph_id}}-reset">Reset</button>
#                                          `;
#             containerElement.appendChild(controlsContainer);
#
#         // Add event listeners for the custom controls
#         document.getElementById(`${{graph_id}}-zoom-in`).addEventListener("click", () => panZoom.zoomIn());
#         document.getElementById(`${{graph_id}}-zoom-out`).addEventListener("click", () => panZoom.zoomOut());
#         document.getElementById(`${{graph_id}}-reset`).addEventListener("click", () => {{
#                 panZoom.resetZoom();
#                 panZoom.center();
#                 }});
#             }}, 500); // Short delay to ensure SVG rendering is complete
#         }}
#
#         // Load Mermaid.js if not already loaded
#         if (typeof mermaid === "undefined") {{
#             loadScript("https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js", () => {{
#             console.log("Mermaid.js loaded successfully");
#             initializeMermaid(graph_id); // Call initialization after Mermaid.js is loaded
#         }});
#             }} else {{
#                 initializeMermaid(graph_id); // If Mermaid.js is already loaded, proceed immediately
#             }}
#         }});
#     </script>
#
#
#     </body>
#     </html>
#     """

    

        # mermaid_html = f"""
    #     <h2 style="text-align: center; color: #007acc; margin-bottom: 16px;">
    #         {escaped_header}
    #     </h2>
#         """ if header else ""
# #     <div class="diagram-container" style="width: 90%; max-width: 800px; margin: auto; padding: 20px; background: white; border: 1px solid #ddd; border-radius: 12px; position: relative;">
# #         {header_html}
# #         <div class="pan-zoom-container" style="width: 100%; height: 500px; overflow: hidden; position: relative; background: #f9f9f9; border: 1px solid #ccc; cursor: grab;">
# #             <div id="{graph_id}" class="mermaid pan-zoom-content" style="position: absolute; transform-origin: 0 0; cursor: grab;">
# #                 {escaped_mermaid_code}
# #             </div>
# #         </div>
# #     </div>
# #     <script src="https://cdn.jsdelivr.net/npm/mermaid/dist/mermaid.min.js"></script>
# #     <script>
# #         // Initialize Mermaid.js and set up pan/zoom
# #         mermaid.initialize({{ startOnLoad: true }});
# #
# #         const container = document.querySelector('.pan-zoom-container');
# #         const content = document.querySelector('.pan-zoom-content');
# #         const rect = content.getBoundingClientRect();
# #
# #         let scale = 1; // Current zoom level
# #         let panX = 0;  // X-axis pan offset
# #         let panY = 0;  // Y-axis pan offset
# #         let isDragging = false;
# #         let startX, startY;
# #
# #         // Helper: Apply transformations
# #         const applyTransform = () => {{
# #             content.style.transform = `translate(${{panX}}px, ${{panY}}px) scale(${{scale}})`;
# #         }};
# #
# #         // Helper: Center the diagram and fit it to the container
# #         const centerAndFitDiagram = () => {{
# #             const containerRect = container.getBoundingClientRect();
# #             const rect = content.getBoundingClientRect();
# #
# #             scale = Math.min(
# #                 containerRect.width / rect.width,
# #                 containerRect.height / rect.height
# #             );
# #
# #             panX = (containerRect.width - rect.width * scale) / 2;
# #             panY = (containerRect.height - rect.height * scale) / 2;
# #
# #             applyTransform();
# #             console.log("Diagram centered and fitted.");
# #         }};
# #
# #         // Add zoom functionality
# #         container.addEventListener('wheel', function(event) {{
# #             event.preventDefault();
# #             const zoomSpeed = 0.1;
# #             const previousScale = scale;
# #
# #             if (event.deltaY < 0) {{
# #                 scale = Math.min(scale + zoomSpeed, 4);
# #             }} else {{
# #                 scale = Math.max(scale - zoomSpeed, 0.5);
# #             }}
# #
# #             const zoomRatio = scale / previousScale;
# #             panX -= (event.clientX - container.getBoundingClientRect().left) * (zoomRatio - 1);
# #             panY -= (event.clientY - container.getBoundingClientRect().top) * (zoomRatio - 1);
# #
# #             applyTransform();
# #         }});
# #
# #         // Add drag functionality for panning
# #         container.addEventListener('mousedown', function(event) {{
# #             isDragging = true;
# #             startX = event.clientX - panX;
# #             startY = event.clientY - panY;
# #             container.style.cursor = "grabbing";
# #         }});
# #
# #         container.addEventListener('mousemove', function(event) {{
# #             if (!isDragging) return;
# #
# #             panX = event.clientX - startX;
# #             panY = event.clientY - startY;
# #
# #             applyTransform();
# #         }});
# #
# #         container.addEventListener('mouseup', function() {{
# #             isDragging = false;
# #             container.style.cursor = "grab";
# #         }});
# #
# #         container.addEventListener('mouseleave', function() {{
# #             isDragging = false;
# #             container.style.cursor = "grab";
# #         }});
# #
# #         // Center diagram after rendering by Mermaid
# #         setTimeout(centerAndFitDiagram, 200);
# #     </script>
# # """
#
#     return mermaid_html



def save_mermaid_html(
    title: str, mermaid_str: str, folder: str = EGERIA_MERMAID_FOLDER
):
    """Save a Mermaid diagram to a file"""
    if not os.path.exists(folder):
        os.makedirs(folder)
    mermaid_file = os.path.join(folder, title + ".html")

    payload = construct_mermaid_web(mermaid_str)

    with open(mermaid_file, "w") as f:
        f.write(payload)
    return mermaid_file


def save_mermaid_graph(title, mermaid_str, folder: str = EGERIA_MERMAID_FOLDER):
    """Save a Mermaid diagram to a file"""
    if not os.path.exists(folder):
        os.makedirs(folder)
    mermaid_file = os.path.join(folder, title + ".html")

    title, mermaid_code = parse_mermaid_code(mermaid_str)

    html_prefix = """
    <html>
        <head>
          <style type="text/css">
            #mySvgId {
            width: 100%;
            height: 100%;
            overflow: hidden;
            border: 1px solid #ccc;
            position: relative;
            margin-bottom: 10px;
            }
            svg {
            cursor: grab;
            }
        
          </style>
        </head>
        
        <body>
          <div id="graphDiv"></div>
          <script src="https://bumbu.me/svg-pan-zoom/dist/svg-pan-zoom.min.js"></script>
          <script type="module">
            import mermaid from 'https://cdn.jsdelivr.net/npm/mermaid@11/dist/mermaid.esm.min.mjs';
        
            mermaid.initialize({startOnLoad: false});
            await mermaid.run({
              querySelector: '.mermaid',
              postRenderCallback: (id) => {
                const container = document.getElementById("diagram-container");
                const svgElement = container.querySelector("svg");
        
                // Initialize Panzoom
                const panzoomInstance = Panzoom(svgElement, {
                  maxScale: 5,
                  minScale: 0.5,
                  step: 0.1,
                });
        
                // Add mouse wheel zoom
                container.addEventListener("wheel", (event) => {
                  panzoomInstance.zoomWithWheel(event);
                });
              }
            });
        
        
            const drawDiagram = async function () {
              const element = document.querySelector('#graphDiv');
              const graphDefinition = `
    """

    html_postfix = r"""`;
              const {svg} = await mermaid.render('mySvgId', graphDefinition);
              element.innerHTML = svg.replace(/( )*max-width:( 0-9\.)*px;/i, '');
        
              var doPan = false;
              var eventsHandler;
              var panZoom;
              var mousepos;
        
              eventsHandler = {
                haltEventListeners: ['mousedown', 'mousemove', 'mouseup']
        
                , mouseDownHandler: function (ev) {
                  if (event.target.className == "[object SVGAnimatedString]") {
                    doPan = true;
                    mousepos = {x: ev.clientX, y: ev.clientY}
                  }
                  ;
                }
        
                , mouseMoveHandler: function (ev) {
                  if (doPan) {
                    panZoom.panBy({x: ev.clientX - mousepos.x, y: ev.clientY - mousepos.y});
                    mousepos = {x: ev.clientX, y: ev.clientY};
                    window.getSelection().removeAllRanges();
                  }
                }
        
                , mouseUpHandler: function (ev) {
                  doPan = false;
                }
        
                , init: function (options) {
                  options.svgElement.addEventListener('mousedown', this.mouseDownHandler, false);
                  options.svgElement.addEventListener('mousemove', this.mouseMoveHandler, false);
                  options.svgElement.addEventListener('mouseup', this.mouseUpHandler, false);
                }
        
                , destroy: function (options) {
                  options.svgElement.removeEventListener('mousedown', this.mouseDownHandler, false);
                  options.svgElement.removeEventListener('mousemove', this.mouseMoveHandler, false);
                  options.svgElement.removeEventListener('mouseup', this.mouseUpHandler, false);
                }
              }
              panZoom = svgPanZoom('#mySvgId', {
                zoomEnabled: true
                , controlIconsEnabled: true
                , fit: 1
                , center: 1
                , customEventsHandler: eventsHandler
              })
            };
            await drawDiagram();
          </script>
        </body>    
    """

    payload = html_prefix + mermaid_code + html_postfix

    with open(mermaid_file, "w") as f:
        f.write(payload)
    return mermaid_file


def generate_process_graph(
    process_guid: str,
    view_server: str = EGERIA_VIEW_SERVER,
    url: str = EGERIA_VIEW_SERVER_URL,
    user_id: str = EGERIA_USER,
    user_pass: str = EGERIA_USER_PASSWORD,
) -> str:
    """Generate Mermaid Markdown text reflecting the Egeria process graph identified by the GUID

    Parameters
        ----------
        process_guid: str
            The identity of the progress to generate a graph of.

        Returns
        -------
        str

        A Mermaid markdown string

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

    """
    console = Console()
    a_client = AutomatedCuration(view_server, url, user_id, user_pass)
    token = a_client.create_egeria_bearer_token()
    try:
        start_time = time.perf_counter()
        response = a_client.get_gov_action_process_graph(process_guid)
        duration = time.perf_counter() - start_time

        if type(response) is dict:
            process_steps: [dict]
            step_links: [dict] = []
            gov_process_qn = response["governanceActionProcess"]["processProperties"][
                "qualifiedName"
            ]
            gov_process_dn = response["governanceActionProcess"]["processProperties"][
                "displayName"
            ]

            md = f"\n---\ntitle: {gov_process_dn}\n---\nflowchart LR\n"
            header = '%%{init: {"flowchart": {"htmlLabels": false}} }%%\n'
            md += header
            element = response["firstProcessStep"]["element"]
            qname = element["processStepProperties"]["qualifiedName"]
            dname = element["processStepProperties"]["displayName"]
            domain_id = element["processStepProperties"]["domainIdentifier"]
            guid = element["elementHeader"]["guid"]
            wait = element["processStepProperties"]["waitTime"]
            ignore_mult_trig = element["processStepProperties"][
                "ignoreMultipleTriggers"
            ]
            link = response["firstProcessStep"]["linkGUID"]

            md = f'{md}\nStep1("`**{dname}**\n*nwait_time*: {wait}\n*nmult_trig*: {ignore_mult_trig}`")'
            process_steps = {
                qname: {
                    "step": "Step1",
                    "displayName": dname,
                    "guid": guid,
                    "domain": domain_id,
                    "ignoreMultTrig": ignore_mult_trig,
                    "waitTime": wait,
                    "link_guid": link,
                }
            }
            next_steps = response.get("nextProcessSteps", None)
            if next_steps is not None:
                i = 1
                for step in next_steps:
                    i += 1
                    qname = step["processStepProperties"]["qualifiedName"]
                    dname = step["processStepProperties"]["displayName"]
                    wait = step["processStepProperties"]["waitTime"]
                    step = f"Step{i}"
                    md = f'{md}\n{step}("`**{dname}**\n*wait_time*: {wait}\n*mult_trig*: {ignore_mult_trig}`")'
                    process_steps.update(
                        {
                            qname: {
                                "step": step,
                                "displayName": dname,
                                "guid": guid,
                                "domain": domain_id,
                                "ignoreMultTrig": ignore_mult_trig,
                                "waitTime": wait,
                            }
                        }
                    )  # process_steps.append({qname: {"step": step,"displayName": dname, "waitTime": wait}})
            # print(md)
            # Now process the links
            process_step_links = response.get("processStepLinks", None)
            if process_step_links is not None:
                for slink in process_step_links:
                    prev_step_name = slink["previousProcessStep"]["uniqueName"]
                    next_step_name = slink["nextProcessStep"]["uniqueName"]
                    next_step_link_guid = slink["nextProcessStepLinkGUID"]
                    guard = slink["guard"]
                    mandatory_guard = slink["mandatoryGuard"]
                    # print(f"\n\n Links: prev_step: {prev_step_name}\t next_step: {next_step_name}\t next_step_link:
                    # {next_step_link_guid}\t Guard: {guard}\t mandatory_guard: {mandatory_guard}")
                    step_links.append(
                        {
                            next_step_link_guid: {
                                "prev_step_name": prev_step_name,
                                "next_step_name": next_step_name,
                                "guard": guard,
                                "mandatory_guard": mandatory_guard,
                            }
                        }
                    )
                    step_p = process_steps[prev_step_name]["step"]
                    step_n = process_steps[next_step_name]["step"]
                    if mandatory_guard:
                        link = f"Mandatory:{guard}"
                    else:
                        link = guard
                    md = f"{md}\n{step_p}-->|{link}|{step_n}"
                i = 1

            return md

        elif type(response) is str:
            console.log("\n\n" + response)
        assert True

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        console.print_exception(show_locals=True)
        assert False, "Invalid request"

    finally:
        a_client.close_session()
